@@ -14,7 +14,86 @@ use super::*;
 
 mod gitlab;
 
+#[derive(Clone, Debug)]
+pub(super) struct RegisterCluster {
+    helm: k8s::Helm,
+    namespace: String,
+    states: Option<Vec<v0::StateName>>,
+    default_state: Option<String>,
+    claim_unowned_states: bool,
+    cleanup_grace: String,
+}
+
+impl RegisterCluster {
+    pub(super) fn new(
+        helm: k8s::Helm,
+        namespace: String,
+        states: Option<Vec<v0::StateName>>,
+        default_state: Option<String>,
+        claim_unowned_states: bool,
+        cleanup_grace: String,
+    ) -> Self {
+        Self {
+            helm,
+            namespace,
+            states,
+            default_state,
+            claim_unowned_states,
+            cleanup_grace,
+        }
+    }
+
+    pub(super) fn states(&self) -> Option<&[v0::StateName]> {
+        self.states.as_deref()
+    }
+}
+
 impl StateHub {
+    pub(super) async fn register_cluster_helper(
+        &self,
+        kubectl: k8s::Kubectl,
+        provider: Option<v0::Provider>,
+        locations: &[Location],
+        register_cluster: RegisterCluster,
+    ) -> anyhow::Result<()> {
+        let provider = if let Some(provider) = provider {
+            provider
+        } else {
+            kubectl.get_cluster_provider().await?
+        };
+
+        let cluster_name = kubectl.cluster_name();
+        let cluster = self
+            .api
+            .register_cluster(&cluster_name, provider, locations)
+            .await
+            .map(Quiet)?;
+
+        self.inform(format_args!(
+            "Registering {:#} cluster {} in {}",
+            provider,
+            cluster.name,
+            locations.show(),
+        ))?;
+
+        k8s::validate_namespace(&register_cluster.namespace).await?;
+
+        self.setup_cluster_token_helper(&cluster, &register_cluster)
+            .await?;
+        self.setup_configmap_helper(&kubectl, &cluster, &register_cluster)
+            .await?;
+
+        if register_cluster.claim_unowned_states {
+            self.claim_unowned_states_helper(&cluster, &register_cluster)
+                .await?;
+        }
+
+        self.helm_install_helper(&cluster, &register_cluster)
+            .await?;
+
+        cluster.print(&self.stdout, self.json)
+    }
+
     pub(super) async fn add_location_helper(
         &self,
         state: &v0::State,
@@ -222,15 +301,19 @@ or run following commands before running 'statehub register-cluster`
         &self,
         kubectl: &k8s::Kubectl,
         cluster: &v0::Cluster,
-        helm: &k8s::Helm,
-        cleanup_grace: &str,
+        register_cluster: &RegisterCluster,
     ) -> anyhow::Result<()> {
-        let namespace = helm.namespace();
-        let default_state = helm.default_state().unwrap_or("");
+        let default_state = register_cluster.default_state.as_deref().unwrap_or("");
         let api = self.api.url("");
 
         kubectl
-            .store_configmap(namespace, &cluster.name, default_state, &api, cleanup_grace)
+            .store_configmap(
+                &register_cluster.namespace,
+                &cluster.name,
+                default_state,
+                &api,
+                &register_cluster.cleanup_grace,
+            )
             .await?;
 
         Ok(())
@@ -239,25 +322,24 @@ or run following commands before running 'statehub register-cluster`
     pub(super) async fn setup_cluster_token_helper(
         &self,
         cluster: &v0::Cluster,
-        helm: &k8s::Helm,
+        register_cluster: &RegisterCluster,
     ) -> anyhow::Result<()> {
         let token = self.api.issue_cluster_token(&cluster.name).await?;
         self.verbosely(format!("Issued token {} for {}", token.token, cluster))?;
-        let namespace = helm.namespace();
-        k8s::store_cluster_token(namespace, &token.token).await?;
+        k8s::store_cluster_token(&register_cluster.namespace, &token.token).await?;
         Ok(())
     }
 
     pub(super) async fn claim_unowned_states_helper(
         &self,
         cluster: &v0::Cluster,
-        states: Option<Vec<v0::StateName>>,
+        register_cluster: &RegisterCluster,
     ) -> anyhow::Result<()> {
-        if let Some(states) = states {
+        if let Some(ref states) = register_cluster.states {
             for state in states {
-                if self.api.get_state(&state).await?.owner.is_none() {
+                if self.api.get_state(state).await?.owner.is_none() {
                     self.verbosely(format!("Claiming ownership of state {}", state))?;
-                    self.api.set_owner(&state, &cluster.name).await?;
+                    self.api.set_owner(state, &cluster.name).await?;
                 }
             }
         }
@@ -315,10 +397,13 @@ or run following commands before running 'statehub register-cluster`
 
     pub(super) async fn helm_install_helper(
         &self,
-        helm: &k8s::Helm,
         cluster: &v0::Cluster,
+        register_cluster: &RegisterCluster,
     ) -> anyhow::Result<()> {
-        let (stdout, stderr) = helm.execute(cluster).await?;
+        let (stdout, stderr) = register_cluster
+            .helm
+            .execute(cluster, &register_cluster.namespace)
+            .await?;
 
         self.verbosely(stdout)?;
         self.error(stderr)?;
